/**********************************************************************
** Copyright (C) 2002 Olaf Lueg.  All rights reserved.
** Copyright (C) 2002 KMerlin Developer Team.  All rights reserved.
**
** This file is part of KMerlin.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://kmerlin.olsd.com/gpl/ for GPL licensing information.
**
** Contact olueg@olsd.de if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#include "cfiletransfer.h"
#include "imchatservice.h"


#include <klocale.h>
#include <kdebug.h>
#include <kurl.h>

#include <time.h>
#include <stdio.h>
#include <math.h>


#include <qhostaddress.h>
#include <qlayout.h>
#include <qpushbutton.h>
#include <qlabel.h>
#include <qtimer.h>
#include <qdatastream.h>
#include <qdir.h>
#include <qprogressbar.h>
#include <qtextbrowser.h>

// TODO Cancel file transfer
// TODO Send signal to the chatwindow ( progress and finish )
// TODO Check the filesize between invite command and received file size (FIL)
// TODO Not accepting command doesn't work
// TODO Include more error handling

CTransferDialog::CTransferDialog( QWidget *parent, const char *name ) : CTransferDlg( parent, name )
{
	connect( acceptB, SIGNAL( clicked() ), this, SIGNAL( accept() ) );
	connect( rejectB, SIGNAL( clicked() ), this, SIGNAL( cancel() ) );
	connect( cancelB, SIGNAL( clicked() ), this, SIGNAL( cancel() ) );
	count = 1;
	QTimer::singleShot( 1000, this, SLOT( slotCount() ) );
}

CTransferDialog::~CTransferDialog()
{}
void CTransferDialog::slotCount()
{
	int t = 60 - count;
	if ( t == 0 )
		emit cancel();
	else
		QTimer::singleShot( 1000, this, SLOT( slotCount() ) );
	++count;
}
void CTransferDialog::setProgress( uint progress )
{
	m_progress->setProgress( progress );
}
void CTransferDialog::setText( const char *text )
{
	m_browser->append( text );
}

CServerSocket::CServerSocket( const QHostAddress & address, Q_UINT16 port, int backlog, QObject * parent, const char * name ) :
		QServerSocket( address, port, backlog, parent, name )
{}

void CServerSocket::newConnection( int socket )
{
	emit established( socket );
}


CFileTransfer::CFileTransfer()
		: QObject()
{
	serverSocket =0;
	bytesWritten =0;
	totalBytes = 0;
	dlg = new CTransferDialog(0,"TR");
	dlg->setProgress( 0 );
	dlg->hide();
	connect( dlg, SIGNAL( cancel() ), this, SLOT( slotCancel() ) );
	connect( dlg, SIGNAL( accept() ), this, SLOT( slotAccept() ) );

}


CFileTransfer::~CFileTransfer()
{
	if( dlg )
		delete dlg;
	switchboard->files.remove( inviteCookie );
	if( serverSocket )
		delete serverSocket;
}

void CFileTransfer::sendCommand( Command c )
{}

void CFileTransfer::sendFile( KURL url )
{
	//QHostAddress ad;
	//ad.setAddress("192.168.1.77");
	dlg->show();
	dlg->acceptB->hide() ;
	dlg->rejectB->hide();
	debug( url.path( false ) );
	sendfile.setName( url.path( false ) );
	filename = url.fileName( false );
	dlg->setText( i18n("<center>Send file</center><br>Filename: %1<br>File size: %2 KB<br>Waiting for %3 ....").arg( filename ).arg( sendfile.size()/1024).arg( switchboard->msgHandle ) ); 
	mode = handshake;
	serverSocket = new CServerSocket( switchboard->msgSocket->address(), 6891 );
	//serverSocket = new CServerSocket( ad, 6891 );// TODO thats for my network, rplace it with above
	if ( !serverSocket->ok() ) {
		delete serverSocket;
		dlg->setText( "Server socket error" );
		return ;
	}
			
		bool open = sendfile.open(IO_ReadOnly );
		if( !open ){
		 dlg->setText( "File not open<br>File transfer has canceled !!!"); 
		 return;
		}
	connect( serverSocket, SIGNAL( established( int ) ), this, SLOT( slotNewConnection( int ) ) );
	time_t Tr_ID = time( ( time_t * ) NULL );
	QString message = "MIME-Version: 1.0\r\n";
	message += "Content-Type: text/x-msmsgsinvite; charset=UTF-8\r\n\r\n";
	message += "Application-Name: File Transfer\r\n";
	message += "Application-GUID: {5D3E02AB-6190-11d3-BBBB-00C04F795683}\r\n";
	message += "Invitation-Command: INVITE\r\n";
	message += "Invitation-Cookie: " + inviteCookie + "\r\n";
	message += "Application-File: "+ filename + "\r\n";
	message += QString("Application-FileSize: %1\r\n").arg( sendfile.size() );
	QString cmd;
	cmd.sprintf( "MSG %lu N %d\r\n", Tr_ID, message.length() );
	cmd += message;
	switchboard->msgSocket->writeBlock( cmd, cmd.length() );
	debug( cmd );
}

void CFileTransfer::sendFileInfo()
{
	QString message;
	time_t Tr_ID = time( ( time_t * ) NULL );
	dlg->setText( i18n("<br>%1 has accepted the file transfer").arg( switchboard->msgHandle ) );
	message = QString("MIME-Version: 1.0\r\n"
						"Content-Type: text/x-msmsgsinvite; charset=UTF-8\r\n\r\n"
						"Invitation-Command: ACCEPT\r\n"
						"Invitation-Cookie: %1\r\n"
						"IP-Address: %2\r\n"
						"Port: %3\r\n"
						"AuthCookie: %4\r\n"
						"Launch-Application: FALSE\r\n"
						"Request-Data: IP-Address:\r\n").arg( inviteCookie ).arg( switchboard->msgSocket->address().toString() ).arg("6891").arg( fileCookie ); // TODO replace it with getIP and port
	QString cmd;
	cmd.sprintf( "MSG %lu N %d\r\n", Tr_ID, message.length() );
	cmd += message;
	switchboard->msgSocket->writeBlock( cmd, cmd.length() );
}

void CFileTransfer::slotNewConnection( int socket )
{
	sendSocket.setSocket( socket );
	connect( &sendSocket, SIGNAL( readyRead() ), this, SLOT( slotSendReadyRead() ) );
	connect( &sendSocket, SIGNAL( bytesWritten( int ) ), this, SLOT( slotBytesWritten( int ) ) );
	delete serverSocket;
	serverSocket =0;
}

void CFileTransfer::slotSendReadyRead()
{
	QString message;
	QString command = sendSocket.readLine();
	debug( command );
	if( command.contains( "VER MSNFTP" ) ){
		message = "VER MSNFTP\r\n";
		sendSocket.writeBlock( message, message.length() );
	}
	else if( command.contains( "USR") ){
		message.sprintf( "FIL %d\r\n",sendfile.size() );
		sendSocket.writeBlock( message, message.length() );
	}
	else if( command.contains( "TFR" ) ){
		dlg->setText( i18n("Transfer started ...") );
		// start send
		mode = send;
		writeFile();
	}
	else if( command.contains("BYE") ){
		debug("Finished");// TODO send signal
		delete this;
	}
	else{// something like cancel
		debug("Error"); // TODO send signal
		delete this;
	}
}

void CFileTransfer::writeFile()
{
	QByteArray ba( 2045 ) ;
	uint result = sendfile.readBlock( ba.data(), ba.size() );
	ba.resize( result );
	QByteArray buffer( 3 );
	buffer[0] = '0';
	buffer[1] = static_cast<int>( fmod( static_cast<double>( result ), 256.0 ) );
	buffer[2] = static_cast<int>( floor( static_cast<double>( result ) / 256.0 ) );
	bytesWritten = result +3;
	sendSocket.writeBlock( buffer, 3 );
	sendSocket.writeBlock( ba, result );
}

void CFileTransfer::slotBytesWritten( int bytes )
{
	if( mode == send ){
		bytesWritten -= bytes;
		totalBytes += bytes;
		dlg->setProgress( ( (totalBytes* 100 )/sendfile.size() )  ) ;
		if( bytesWritten == 0 )
			writeFile();
	}
}

void CFileTransfer::acceptFile()
{
	dlg->show();
	dlg->cancelB->hide();
	dlg->setText( i18n("<center>Accept file transfer from %1 ?</center>"
										"Filename: %2"
										"File size: %3 KB").arg( switchboard->msgHandle).arg( filename ).arg( filesize/1024 ) );
}
void CFileTransfer::slotAccept()
{
	QString message, command;
	time_t Tr_ID = time( ( time_t * ) NULL );
	message = "MIME-Version: 1.0\r\nContent-Type: text/x-msmsgsinvite; charset=UTF-8\r\n\r\nInvitation-Command: ACCEPT\r\n";
	message += "Invitation-Cookie: " + inviteCookie + "\r\nLaunch-Application: FALSE\r\nRequest-Data: IP-Address:\r\n";
	command.sprintf( "MSG %lu N %d\r\n", Tr_ID, message.length() );
	command += message;
	switchboard->msgSocket->writeBlock( command, command.length() );
	dlg->acceptB->hide();
	dlg->rejectB->hide();
	dlg->cancelB->show();
	dlg->setText( i18n("<br>Transfer has accepted" ) );
}

void CFileTransfer::slotCancel()
{
	QString message, command;
	time_t Tr_ID = time( ( time_t * ) NULL );
	message = "MIME-Version: 1.0\r\nContent-Type: text/x-msmsgsinvite; charset=UTF-8\r\n\r\nInvitation-Command: CANCEL\r\n";
	message += "Invitation-Cookie: " + inviteCookie + "\r\nCancel-Code: REJECT\r\n";
	command.sprintf( "MSG %lu N %d\r\n", Tr_ID, message.length() );
	command += message;
	switchboard->msgSocket->writeBlock( command, command.length() );
	delete dlg;
	delete this;
}
/*! get the current IP
*/
void CFileTransfer::getIP()
{
	localIP = ( ( QHostAddress ) switchboard->msgSocket->address() ).toString();
}

void CFileTransfer::startReceive()
{
	receiveSocket = new QSocket;
	mode = handshake;
	QDir::setCurrent( QDir::homeDirPath() + "/kmerlin/incoming" ) ;
	file = fopen( filename, "w" );
	packetSize = 0;
	recBytes = 0;
	connect( receiveSocket, SIGNAL( readyRead() ), this, SLOT( slotDataReceived() ) );
	connect( receiveSocket, SIGNAL( error( int ) ), this, SLOT( slotSocketError( int ) ) );
	receiveSocket->connectToHost( clientIP, port.toUInt() );
	//receiveSocket->connectToHost( "192.168.1.78", port.toUInt() ); // TODO only for my network, use above
	QString command;
	command.sprintf( "VER MSNFTP\r\n" );
	dlg->setText( i18n("<br>Connecting to %1").arg( switchboard->msgHandle ) );
	receiveSocket->writeBlock( command, command.length() );

}

void CFileTransfer::slotSocketError( int socketError )
{
	Q_ASSERT( socketError > 0 );
	delete this;
}

/*! receiving a file
* TODO include error handling and transfer stop
*/
void CFileTransfer::slotDataReceived()
{
	QString str, command;
	int size1, size2;
	QCString buffer;
	if ( mode == receive ) {
		while ( receiveSocket->bytesAvailable() ) {
			if ( packetSize == 0 ) {
				code = receiveSocket->getch();
				if ( code == 0 ) {
					size1 = receiveSocket->getch();
					size2 = receiveSocket->getch();
					packetSize = size1 + 256 * size2;
				}
			}
			if ( code == 0 ) {
				QByteArray ba( packetSize ) ;
				uint result = receiveSocket->readBlock( ba.data(), ba.size() );
				recBytes += result;
				packetSize -= result;
				ba.resize( result );
				fwrite( ba, result, 1, file );
				dlg->setProgress( (recBytes*100)/filesize );
				if ( recBytes >= filesize ) {
					fclose( file );
					sendFinished();
					return ;
				}
			} // TODO code == 1 - something like cancel
		} // end while
	} else { // handshake
		buffer.resize( receiveSocket->bytesAvailable() );
		receiveSocket->readBlock( buffer.data(), receiveSocket->bytesAvailable() );
		if ( buffer.contains( "VER MSNFTP" ) ) {
			command.sprintf( "USR %s %s\r\n", handle.latin1(), fileCookie.latin1() );
			receiveSocket->writeBlock( command, command.length() );
		} else if ( buffer.contains( "FIL" ) ) {
			QString size = buffer.right( buffer.length() - buffer.find( "FIL" ) - 4 );
			filesize = size.left( size.find( "\r\n" ) ).toUInt();
			command.sprintf( "TFR\r\n" );
			receiveSocket->writeBlock( command, command.length() );
			dlg->setText( i18n("Transfer has started ....") );
			mode = receive;
		}
	}
}

void CFileTransfer::writeData()
{}

void CFileTransfer::sendFinished()
{
	QString command = "BYE 16777989\r\n";
	debug( "Transfer finished" );
	debug( "Received bytes: %i filesize: %i", recBytes, filesize );
	receiveSocket->writeBlock( command, command.length() );
	delete this;
}

