
#ifndef CFILETRANSFER_H
#define CFILETRANSFER_H


#include <qobject.h>
#include <qsocket.h>
#include <qwidget.h>
#include <stdint.h>
#include "ctransferdlg.h"
#include <qserversocket.h>
#include <qfile.h>

class IMChatService;
class QPushButton;
class QLabel;
class QFile;

class CServerSocket: public QServerSocket
{
	Q_OBJECT
public:
	CServerSocket( const QHostAddress & address, Q_UINT16 port, int backlog = 1, QObject * parent = 0, const char * name = 0 );
	~CServerSocket() {}
protected:
	void newConnection( int socket );
signals:
	void established( int socket );
};

class CTransferDialog : public CTransferDlg
{
	Q_OBJECT
public:
	CTransferDialog( QWidget *parent, const char *name);
	~CTransferDialog();
	void setText( const char *text);
signals:
	void cancel();
	void accept();
private:
	int count;
private slots:
	void slotCount();
public slots:
	void setProgress( uint progress );
};

class KURL;
class CFileTransfer : public QObject
{
Q_OBJECT
public:
  CFileTransfer();
  ~CFileTransfer();
	enum Command { accept, cancel, invite };
	enum Mode{ handshake, receive, send };
	enum FileMode { Send, Receive };
	Mode mode;
	FileMode fileMode;
	
public:
	void setSwitchboard( IMChatService *so ){ switchboard = so ;}
	void setIP( const char *IP ){ clientIP = IP; }
	void setPort( const char *Port ){ port = Port; }
	void setFileName( const char *Filename ){ filename = Filename; }
	void setFileSize( uint32_t Filesize ){ filesize = Filesize; }
	void setInviteCookie( const char *Cookie ){ inviteCookie = Cookie; }
	void setFileCookie( const char *Cookie ){ fileCookie = Cookie; }
	void setHandle( const char *myHandle){ handle = myHandle ;}
	void startReceive();
	void getIP();
	void acceptFile();
	void sendFile( KURL url);
	void sendFileInfo();
	
protected:
	void sendCommand( Command );
private:
	void writeData();
	void sendFinished();
	void writeFile();
	
private slots:
	void slotDataReceived();
	void slotAccept();
	void slotCancel();
	void slotSocketError( int socketError);
	void slotNewConnection( int socket );
	void slotSendReadyRead();
	void slotBytesWritten( int bytes );
	
private:
	QString port;
	QString localIP;
	QString clientIP;
	uint32_t filesize;
	QString filename;
	QString inviteCookie;
	QString fileCookie;
	QString handle;
	QSocket *receiveSocket;
	QSocket sendSocket;
	IMChatService *switchboard;
	uint32_t recBytes;
	uint32_t totalBytes;
	uint16_t packetSize;
	int bytesWritten;
	int code;
	FILE *file;
	QFile sendfile;
	CServerSocket *serverSocket;
	CTransferDialog *dlg;
	
};

#endif
