/**********************************************************************
** Copyright (C) 2002 Olaf Lueg.  All rights reserved.
** Copyright (C) 2002 KMerlin Developers Team.  All rights reserved.
**
** This file is part of KMerlin.
**
** This file may be distributed and/or modified under the terms of the
** GNU General Public License version 2 as published by the Free Software
** Foundation and appearing in the file LICENSE included in the
** packaging of this file.
**
** This file is provided AS IS with NO WARRANTY OF ANY KIND, INCLUDING THE
** WARRANTY OF DESIGN, MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.
**
** See http://kmerlin.olsd.com/gpl/ for GPL licensing information.
**
** Contact olueg@olsd.de if any conditions of this licensing are
** not clear to you.
**
**********************************************************************/

#include "kmservice.h"
#include "kmcontact.h"
#include "kmgroup.h"
#include "kmerlin.h"
#include "imchatservice.h"
#include "qcategorywidget.h"
#include "authdlg.h"

// qt include
#include <qregexp.h>
#include <qcheckbox.h>
#include <qlabel.h>
#include <qpushbutton.h>
#include <qtimer.h>

// kde include
#include <kdebug.h>
#include <klocale.h>
#include <kmessagebox.h>
#include <kmdcodec.h>
#include <ktempfile.h>
#include <kapp.h>
#include <kstddirs.h>

/**/
KMService::KMService()
		: QObject()
{
	theApp = KMerlin::getInstance();
	connected = false;
	_serial = "0";
	_silent = false; // remove later
	autoRename = true; // set new nick names in all lists
	connect( this, SIGNAL( signalConnected( bool ) ), KMerlin::getInstance(), SLOT( slotConnected( bool ) ) );
	lastPing = true;
	socket = 0L;
	mailCount =0;
}
/**/
KMService::~KMService()
{}
/**/
void KMService::cleanUp()
{
	//clean up all lists , but make sure that chat window has cleaned before
	theApp->groupList.clear();
	theApp->contactList.clear();
	theApp->groupWidget->cleanUp();
	theApp->onlineAction.clear();
	theApp->offlineAction.clear();
	//  theApp->onlineWidget->cleanUp();
}
/**/
void KMService::slotDataReceived( )
{
	QString data;
	if ( socket->canReadLine() ) {
		data = socket->readLine();
	} else {
		return ;
	}
	data = QString::fromUtf8( data );
	kdDebug() << data << endl;
	if ( data.left( 3 ) == "911" ) {
		emit signalConnected( false );
		if ( !_silent ) {
			KMessageBox::error( 0, i18n( "Authentication failed" ) );
			kdDebug() << "Connection error: Authentication failed!" << endl;
		}
		return ;
	}
	if ( data.left( 3 ) == "280" || data.left( 3 ) == "281" ) {
		KMessageBox::error( 0, i18n( "Sorry!\nThere is a problem with the server." ) );
		return ;
	}
	if ( !connected ) {
		if ( data.left( 3 ) == "XFR" ) {
			// new server reconnect
			emit signalStatus( i18n( "Try other server" ) );
			newConnect( data );
			return ;
		}
	}
	if ( data.left( 3 ) == "VER" ) {
		sendServerPolicy();
		return ;
	}
	if ( data.left( 3 ) == "INF" ) {
		emit signalStatus( i18n( "Try to login" ) );
		sendInitialInfo();
		return ;
	}
	if ( data.left( 3 ) == "USR" ) {
		if ( kstr.word( data, 3 ) == "S" ) {
			emit signalStatus( i18n( "Sending user information" ) );
			hashRes = data.right( data.length() - data.findRev( " ", -1, false ) - 1 );
			sendResponseInfo();
		} else {
			receivedNick = kstr.word( data, 4 ).replace( QRegExp( "%20" ), " " ) ;
			emit signalStatus( i18n( "Actual nick name received: %1" ).arg( receivedNick ) ) ;
			sendSerial();
		}
		return ;
	}
	parseCommand( data );
	while ( socket->canReadLine() ) {
		data = socket->readLine();
		data = QString::fromUtf8( data );
		parseCommand( data );
	}
}

/**/
void KMService::parseCommand( QString txt )
{
	QString miss, len;
	char dat[ 1024 ];
	txt = txt.replace( QRegExp( "\r\n" ), "" );
	QString command = txt.left( 3 );
	kdDebug() << txt << endl;
	if ( command == "QNG" ) {
		lastPing = true;
		return ;
	}
	if ( command == "NLN" || command == "ILN" || command == "FLN" ) {
		statusReceived( txt );
		return ;
	}
	if ( command == "LSG" ) {
		groupReceived( txt );
		return ;
	}
	if ( command == "LST" ) {
		contactReceived( txt );
		if ( kstr.word( txt, 2 ) == "RL" ) {
			if ( kstr.word( txt, 4 ) == kstr.word( txt, 5 ) ) {
				// setStatus( "NLN " );
				if ( !connected ) {
					checkRemovedContacts();
					connected = true;
					if ( myNick.isEmpty() )
						myNick = receivedNick;
					if ( myNick != receivedNick )
						changePublicName( myNick );
					emit signalStatus( i18n( "Succesful connected, switching to Online" ) );
					setStatus( "NLN" );
					emit signalConnected( true );
					lastPing = true;
					QTimer::singleShot( 60000, this, SLOT( slotPing() ) );
				}
			}
		}
		return ;
	}
	if ( command == "CHL" ) {
		sendFinalAuthentication( kstr.word( txt, 2 ) );
		return ;
	}
	if ( command == "RNG" ) {
		// SessionID, Address, AuthInfo, handle, publicName
		slotCreateChat( kstr.word( txt, 1 ), kstr.word( txt, 2 ), kstr.word( txt, 4 ), kstr.word( txt, 5 ), kstr.word( txt, 6 ).replace( QRegExp( "%20" ), " " ) );
		return ;
	}
	if ( command == "XFR" ) {
		// Address, AuthInfo
		slotCreateChat( kstr.word( txt, 3 ), kstr.word( txt, 5 ) );
		return ;
	}
	if ( command == "MSG" ) {
		len = kstr.word( txt, 3 );
		socket->readBlock( dat, len.toUInt() );
		miss = QString::fromUtf8( dat );
		kdDebug() << miss << endl;
		if ( miss.contains( "Inbox-Unread:" ) ) {
			//this sends the server if we are going online, contains the unread message count
			miss = miss.right( miss.length() - miss.find( "Inbox-Unread:" ) );
			miss = miss.left( miss.find( "\r\n" ) );
			mailCount = miss.right( miss.length() - miss.find( " " ) - 1 ).toUInt();
			emit newMail( "", mailCount, "" );
			return ;
		}
		if ( miss.contains( "Message-Delta:" ) ) {
			// do not count deletings in trash folder or junk mail
			if ( miss.contains( "Dest-Folder: .!!trAsH" ) ) {
				return ;
			}
			if ( miss.contains( "Src-Folder: ACTIVE" ) ) {
				//this sends the server if mails are deleted, only delete mail from "Posteingang"
				miss = miss.right( miss.length() - miss.find( "Message-Delta:" ) );
				miss = miss.left( miss.find( "\r\n" ) );
				mailCount = mailCount - miss.right( miss.length() - miss.find( " " ) - 1 ).toUInt();
				emit newMail( "", mailCount, "" );
				return ;
			}
		}
		if ( miss.contains( "From-Addr:" ) ) {
			//this sends the server if a new mail has arrived
			kdDebug() << "the hole packet:" << miss << "\n" << endl;
			//Message-URL: /cgi-bin/getmsg?msg=MSG1029401739.3&start=1610592&len=402&curmbox=ACTIVE
			//Post-URL: https://lc1.law13.hotmail.passport.com/ppsecure/domessengerlogin/EN
			//Subject: Hi
			QString subject, postUrl, messageUrl;
			subject = miss;
			postUrl = miss;
			messageUrl = miss;

			// subject
			subject = subject.right( subject.length() - subject.find( "Subject: " ) );
			subject = subject.left( subject.find( "\r\n" ) );
			subject = subject.remove( 0, 9 ); //
			kdDebug() << "Message Subject: " << subject << "\n" << endl;

			// Message-URL
			messageUrl = messageUrl.right( messageUrl.length() - messageUrl.find( "Message-URL: " ) );
			messageUrl = messageUrl.left( messageUrl.find( "\r\n" ) );
			messageUrl = messageUrl.remove( 0, 13 ); //
			kdDebug() << "Message URL: " << messageUrl << "\n" << endl;

			// Post-URL
			postUrl = postUrl.right( postUrl.length() - postUrl.find( "Post-URL: " ) );
			postUrl = postUrl.left( postUrl.find( "\r\n" ) );
			postUrl = postUrl.remove( 0, 10 ); //
			kdDebug() << "Post URL: " << postUrl << "\n" << endl;

			miss = miss.right( miss.length() - miss.find( "From-Addr:" ) );
			miss = miss.left( miss.find( "\r\n" ) );
			mailCount++;
			miss = miss.right( miss.length() - miss.find( " " ) - 1 );
			emit newMail( miss, mailCount, subject );
			return ;
		} else if ( miss.contains( "text/x-msmsgsprofile" ) ) {

			//Hotmail profile
			if ( miss.contains( "MSPAuth:" ) ) {
				QRegExp rx( "MSPAuth: ([A-Za-z0-9$!*]*)" );
				rx.search( miss );
				m_MSPAuth = rx.cap( 1 );
			}
			if ( miss.contains( "sid:" ) ) {
				QRegExp rx( "sid: ([0-9]*)" );
				rx.search( miss );
				m_sid = rx.cap( 1 );
			}
			if ( miss.contains( "kv:" ) ) {
				QRegExp rx( "kv: ([0-9]*)" );
				rx.search( miss );
				m_kv = rx.cap( 1 );
			}
			loginTime = time( ( time_t * ) NULL );
			//write the tmp file

			//m_isHotmailAccount = true;
			//emit hotmailSeted(true);
		}
		return ;
	}
	if ( command == "SYN" ) {
		// this is the current serial on the server, if its different with the own we can get the user list
		//isConnected = true;
		uint serial = kstr.word( txt, 2 ).toUInt( );
		if ( serial != _serial ) {
			//emit newSerial(serial);  // remove all contacts, msn sends a new contact list
			_serial = serial;
		}
		return ;
	}
	if ( command == "RMG" ) { // serial, id
		_serial = kstr.word( txt, 2 );
		KMGroup *group;
		for ( group = theApp->groupList.first(); group != 0; group = theApp->groupList.next() ) {
			if ( group->toolId == kstr.word( txt, 3 ) ) {
				group->remove
				();
				return ;
			}
		}
	}
	if ( command == "REG" ) {
		// groupName, serial, group
		KMGroup * group;
		_serial = kstr.word( txt, 2 );
		for ( group = theApp->groupList.first(); group != 0; group = theApp->groupList.next() ) {
			if ( group->toolId == kstr.word( txt, 3 ) ) {
				group->rename( kstr.word( txt, 4 ).replace( QRegExp( "%20" ), " " ) );
				return ;
			}
		}
		return ;
	}
	if ( command == "ADG" ) {
		// groupName, serial, group
		_serial = kstr.word( txt, 2 );
		new KMGroup( kstr.word( txt, 3 ).replace( QRegExp( "%20" ), " " ), kstr.word( txt, 4 ) );
		return ;
	}
	if ( command == "CHG" ) {
		m_status = kstr.word( txt, 2 );
		emit statusChanged( kstr.word( txt, 2 ) );
		return ;
	}
	if ( command == "OUT" ) {
		if ( kstr.word( txt, 1 ) == "OTH" ) {
			KMessageBox::error( 0, i18n( "You have connected\nfrom an other client" ) );
		}
		//      slotSocketClosed();
		//		emit sessionClosed( kstr.word( txt,1) );
		return ;
	}
	if ( command == "REM" )     // someone is removed from a list
	{
		KMContact * contact;
		_serial = kstr.word( txt, 3 );
		QString handle = kstr.word( txt, 4 );
		for ( contact = theApp->contactList.first(); contact != 0; contact = theApp->contactList.next() )
		{
			if ( contact->getHandle() == handle ) {
				if ( kstr.word( txt, 2 ) == "FL" ) {
					contact->removed( handle, "FL", kstr.word( txt, 5 ) );
					return ;
				} 
				else if( kstr.word( txt, 2) == "BL" ){
					contact->setBlocked( false );
					return ;
				}
				else if( kstr.word( txt, 2 ) == "RL" ){
					contact->cleanUp(); // removes the contact with no info
					return;
				} 
			}
		}
		return ;
	}
	if ( command == "ADD" )     // ####fixme - same as contactReceived
	{
		QString handle = kstr.word( txt, 4 );
		QString nick = kstr.word( txt, 5 ).replace( QRegExp( "%20" ), " " );
		QString list = kstr.word( txt, 2 );
		_serial = kstr.word( txt, 3 );
		QString group = 0L;
		KMContact *contact;
		if ( list == "FL" )
		{
			group = kstr.word( txt, 6 );
			for ( contact = theApp->contactList.first(); contact != 0; contact = theApp->contactList.next() ) {
				if ( contact->getHandle() == handle ) {
					if ( contact->isForward() )
						contact->addToGroup( group );
					else
						contact->setForward( nick, group );
					return ;
				}
			}
			contact = new KMContact( handle, nick );
			contact->setForward( nick, group );
			return ;
		}
		for ( contact = theApp->contactList.first(); contact != 0; contact = theApp->contactList.next() )
		{
			if ( contact->getHandle() == handle ) {
				if ( list == "AL" )
					contact->setAllow( true );
				else if ( list == "BL" )
					contact->setBlocked( true );
				else if ( list == "RL" )
					contact->setReverse( true );
				return ;
			}
		}
		contact = new KMContact( handle, nick );
		// ##  need contact auth #############
		AuthDlg * auth = new AuthDlg( 0, "Authenticate" );
		auth->m_handle->setText( handle );
		connect( auth->btnOK, SIGNAL( clicked() ), this, SLOT( authenticate() ) );
		auth->show();
		if ( list == "AL" )
			contact->setAllow( true );
		else if ( list == "BL" )
			contact->setBlocked( true );
		else if ( list == "RL" )
			contact->setReverse( true );
	} // and ADD
	if ( command == "REA" ) {
		QString handle = kstr.word( txt, 3 );
		QString nick = kstr.word( txt, 4 ).replace( QRegExp( "%20" ), " " );
		QString id = kstr.word( txt, 1 );
		_serial = kstr.word( txt, 2 );
		if ( handle == myHandle ) {
			emit nickChanged( nick );
			myNick = nick;
			return ;
		} else if ( id == "0" ) {
			//do nothing
			return ;
		}
		kdDebug() << "REA command?: " << txt << endl;
	}


}
/**/
void KMService::contactReceived( QString txt )
{
	QString handle = kstr.word( txt, 6 );
	if ( handle == "" ) return ;
	QString nick = kstr.word( txt, 7 ).replace( QRegExp( "%20" ), " " );
	QString list = kstr.word( txt, 2 );
	QString group = 0L;
	if ( kstr.word( txt, 4 ) == kstr.word( txt, 5 ) && list == "FL" )
		emit signalStatus( "Receiving contactlist" );
	KMContact *contact;
	if ( list == "FL" ) {
		group = kstr.word( txt, 8 );
		/*for ( contact = theApp->contactList.first(); contact != 0; contact = theApp->contactList.next() ) {
			if ( contact->getHandle() == handle ) {
				contact->( group );
				contact->setMsnNick( nick );
				
				return ;
			}
		}*/
		contact = new KMContact( handle, nick );
		contact->setForward( nick, group );
		kdDebug() << "New Contact inserted: " << handle << endl;
		return ;
	}
	for ( contact = theApp->contactList.first(); contact != 0; contact = theApp->contactList.next() ) {
		if ( contact->getHandle() == handle ) {
			if ( list == "AL" )
				contact->setAllow( true );
			else if ( list == "BL" )
				contact->setBlocked( true );
			else if ( list == "RL" )
				contact->setReverse( true );
			return ;
		}
	}
	// contact not found! not in FL list
	if ( list != "RL" ) {
		contact = new KMContact( handle, nick );
		if ( list == "AL" )
			contact->setAllow( true );
		else if ( list == "BL" )
			contact->setBlocked( true );
		return ;
	}
	// new contact! open auth dialog
	contact = new KMContact( handle, nick );
	AuthDlg * auth = new AuthDlg( 0, "Authenticate" );
	auth->m_handle->setText( handle );
	connect( auth->btnOK, SIGNAL( clicked() ), this, SLOT( authenticate() ) );
	auth->show();
	if ( list == "AL" )
		contact->setAllow( true );
	else if ( list == "BL" )
		contact->setBlocked( true );
	else if ( list == "RL" )
		contact->setReverse( true );
}

void KMService::statusReceived( QString txt )
{
	KMContact * contact;
	QString handle;
	QString nick;
	QString status;
	if ( txt.left( 3 ) == "NLN" ) {
		handle = kstr.word( txt, 2 );
		nick = kstr.word( txt, 3 ).replace( QRegExp( "%20" ), " " );
		status = kstr.word( txt, 1 );
		if ( handle == myHandle ) {
			emit statusChanged( status );
			return ;
		} else {
			for ( contact = theApp->contactList.first(); contact != 0; contact = theApp->contactList.next() ) {
				if ( contact->getHandle() == handle ) {
					// check status, and nickname. ### only send status notify when new status // check for status
					if ( contact->getNick() != nick ){
						changeNickname( handle, nick ); // if nick is different as conta->nick rename it
						contact->setNickName( nick, true );
					}
					if ( contact->getStatus() != status ) {
						contact->setStatus( status );
						emit statusChanged( handle, status );
					}
					return ;
				}
			}
		}
	} else if ( txt.left( 3 ) == "ILN" ) {
		// initial status
		handle = kstr.word( txt, 3 );
		nick = kstr.word( txt, 4 ).replace( QRegExp( "%20" ), " " );
		status = kstr.word( txt, 2 );
		for ( contact = theApp->contactList.first(); contact != 0; contact = theApp->contactList.next() ) {
			if ( contact->getHandle() == handle ) {
				if ( contact->getNick() != nick ){
					changeNickname( handle, nick );
					contact->setNickName( nick, false );
				}
				contact->setStatus( status );
				return ;
			}
		}
	} else if ( txt.left( 3 ) == "FLN" ) {
		handle = kstr.word( txt , 1 );
		status = "FLN";
		for ( contact = theApp->contactList.first(); contact != 0; contact = theApp->contactList.next() ) {
			if ( contact->getHandle() == handle ) {
				contact->setStatus( status );
				emit statusChanged( handle, "FLN" );
			}
		}
	}
}

/**/
void KMService::groupReceived( QString txt )
{
	KMGroup * group;
	if ( txt.contains( "1 1 0 ~ 0" ) ) {
		group = new KMGroup( i18n( "Friends" ), "0" );
		renameGroup( i18n( "Friends" ), "0" );
		return ;
	}
	group = new KMGroup( kstr.word( txt, 6 ).replace( QRegExp( "%20" ), " " ) , kstr.word( txt, 5 ) );
}
/**/
void KMService::closeService()
{
	QString command = "OUT\r\n";
	socket->writeBlock( command, command.length() );
	kdDebug() << "Send OUT" << endl;
}
void KMService::slotSocketClosed()
{
	if ( !connected ){
		if( socket != 0L ){
			delete socket;
			socket = 0L;
		}
		cleanUp();
		emit signalConnected( false );
		return ;
	}
	kdDebug() << "Connection to MSN server closed!" << endl;
	emit signalConnected( false );
	connected = false;
	cleanUp();
	delete socket;
	socket = 0L;
	_serial = "0";
}
/**/
void KMService::killSocket()
{
	slotSocketClosed();
}
/**/
void KMService::sendFinalAuthentication( QString res )
{
	ID = time( ( time_t * ) NULL );
	res.replace( QRegExp( "\r\n" ), "" );
	QString command;
	QString str3 = res + "Q1P7W2E4J9R8U3S5" ;
	KMD5 context( str3 );
	QString str5 = context.hexDigest( );
	command.sprintf( "QRY %lu msmsgs@msnmsgr.com 32\r\n", ID );
	command += str5 ;
	socket->writeBlock( command, command.length( ) );
}

/**/
void KMService::sendProtocol()
{
	ID = time( ( time_t * ) NULL );
	QString command;
	command.sprintf( "VER %lu MSNP7 MSNP6 MSNP5 MSNP4 CVR0\r\n", ID );
	socket->writeBlock( command, command.length() );
	kdDebug() << "Sending protocol" << endl;
}

/**/
void KMService::newConnect( QString data )
{
	QString port, server;
	data = kstr.word ( data, 3 );
	port = data.right( data.length() - data.findRev( ":" ) - 1 );
	server = data.right( data.length() - data.findRev ( " " ) - 1 );
	server = server.left( server.find( ":" ) );
	kdDebug() << "Connecting to new Server:\nServer: " << server << "\nPort: " << port << endl;
	socket->connectToHost( server, port.toUInt() );
}

/**/
void KMService::sendServerPolicy()
{
	ID = time( ( time_t * ) NULL );
	QString command;
	command.sprintf( "INF %lu\r\n", ID );
	socket->writeBlock( command, command.length() );
	kdDebug() << "Sending server policy" << endl;
}
/**/
void KMService::sendInitialInfo()
{
	md5_tr = time( ( time_t * ) NULL );
	QString command;
	command.sprintf( "USR %lu MD5 I ", md5_tr );
	command += myHandle.utf8() + "\r\n";
	socket->writeBlock( command, command.length() );
	kdDebug() << "Sending initial Authentication" << endl;
}
/**/
void KMService::sendResponseInfo()
{
	if ( hashRes.contains( "\r\n", true ) ) {
		hashRes = hashRes.left( hashRes.find( "\r\n", 0, true ) );
	}
	QString str3 = hashRes + myPassword;
	KMD5 context( str3 );
	QString strcommand1, str5 = context.hexDigest();
	strcommand1.sprintf( "USR %lu ", md5_tr );
	strcommand1 += "MD5 S ";
	strcommand1 += str5 + "\r\n";
	socket->writeBlock( strcommand1, strcommand1.length() );
	kdDebug() << "Send Response Info: " << strcommand1 << endl;
}
/**/
void KMService::sendSerial()
{
	ID = time( ( time_t * ) NULL );
	QString command;
	command.sprintf( "SYN %lu ", ID );
	command += _serial + "\r\n" ;
	kdDebug() << "Send: " << command << endl;
	socket->writeBlock( command, command.length() );
}
/**/
void KMService::setStatus( QString status )
{
	ID = time( ( time_t * ) NULL );
	QString command;
	if ( status == "NLN" ) {
		command.sprintf( "CHG %lu NLN\r\n", ID );
	} else if ( status == "BSY" ) {
		command.sprintf( "CHG %lu BSY\r\n", ID );
	} else if ( status == "BRB" ) {
		command.sprintf( "CHG %lu BRB\r\n", ID );
	} else if ( status == "AWY" ) {
		command.sprintf( "CHG %lu AWY\r\n", ID );
	} else if ( status == "PHN" ) {
		command.sprintf( "CHG %lu PHN\r\n", ID );
	} else if ( status == "LUN" ) {
		command.sprintf( "CHG %lu LUN\r\n", ID );
	} else if ( status == "FLN" ) {
		command.sprintf( "CHG %lu FLN\r\n", ID );
	} else if ( status == "HDN" ) {
		command.sprintf( "CHG %lu HDN\r\n", ID );
	} else if ( status == "IDL" ) {
		command.sprintf( "CHG %lu IDL\r\n", ID );
	}

	socket->writeBlock( command, command.length() );
}
/**/
void KMService::slotConnect()
{
	socket = new QSocket();
	connect( socket, SIGNAL( readyRead() ), this, SLOT( slotDataReceived() ) );
	connect( socket, SIGNAL( error( int ) ), this, SLOT( slotSocketError( int ) ) );
	connect( socket, SIGNAL( connected() ), this, SLOT( slotSocketConnected() ) );
	connect( socket, SIGNAL( connectionClosed() ), this, SLOT( slotSocketClosed() ) );
	//  connect(socket,SIGNAL(hostFound()),this,SLOT(slotHostFound()));
	kdDebug() << "Create new connection to:\nServer: messenger.hotmail.com\nPort:1863\n" << endl;
	socket->connectToHost( "messenger.hotmail.com", 1863 );
}
/**/
void KMService::slotSocketError( int error )
{
	if ( !_silent ) {
		switch ( error ) {
				case 0: {
					KMessageBox::error( 0, i18n( "Connection refused" ) );
					kdDebug() << "Connection error: Connection refused!" << endl;
					break;
				}
				case 1: {
					kdDebug() << "Connection error: Host not found!" << endl;
					KMessageBox::error( 0, i18n( "Host not found" ) );
					break;
				}
				case 2: {
					kdDebug() << "Connection error: Socket error!" << endl;
					KMessageBox::error( 0, i18n( "Socket error" ) );
					break;
				}
		}
	}
	emit signalConnected( false );
	connected = false;
}
/**/
void KMService::slotSocketConnected()
{
	sendProtocol();
}

/** change the nick name for a contact- this change it in all lists
  id = 0 , this will be ignored when receiving the server answer
*/
void KMService::changeNickname( const QString &handle, QString nick )
{
	QString command;
	ID = 0;
	command.sprintf( "REA %lu ", ID );
	command += handle.utf8() + " " + nick.replace( QRegExp( " " ), "%20" ).utf8( ) + "\r\n";
	socket->writeBlock( command, command.length() );
}
/**/
void KMService::changePublicName( QString publicName )
{
	QString command;
	ID = time( ( time_t* ) NULL );
	command.sprintf( "REA %lu ", ID );
	command += myHandle.utf8() + " " + publicName.replace( QRegExp( " " ), "%20" ).utf8( ) + "\r\n";
	socket->writeBlock( command, command.length() );
}
/**/
void KMService::removeContact( const QString &handle, const QString &group, const QString &list )
{
	ID = time( ( time_t * ) NULL );
	QString command;
	if ( list == "FL" ) {
		command.sprintf( "REM %lu FL ", ID );
		command += handle.utf8() + " " + group.utf8() + "\r\n";
	} else if ( list == "AL" ) {
		command.sprintf( "REM %lu AL ", ID );
		command += handle.utf8() + "\r\n";
	} else {
		command.sprintf( "REM %lu BL ", ID );
		command += handle.utf8() + "\r\n";
	}
	socket->writeBlock( command, command.length() );
}
/**/
void KMService::addContact( const QString &handle, QString publicName, const QString &group, const QString &list )
{
	time_t ID = time( ( time_t * ) NULL );
	QString command;
	if ( list == "FL" ) {
		command.sprintf( "ADD %lu FL ", ID );
		command += handle.utf8() + " " + handle.utf8() + " " + group.utf8() + "\r\n";
	} else if ( list == "AL" ) {
		command.sprintf( "ADD %lu AL ", ID );
		command += handle.utf8() + " " + publicName.replace( QRegExp( " " ), "%20" ).utf8() + "\r\n";
	} else {
		command.sprintf( "ADD %lu BL ", ID );
		command += handle.utf8() + " " + publicName.replace( QRegExp( " " ), "%20" ).utf8() + "\r\n";
	}
	socket->writeBlock( command, command.length() );
}
/**/
void KMService::addGroup( QString groupName )
{
	groupName.replace( QRegExp( " " ), "%20" );
	ID = time( ( time_t * ) NULL );
	QString command;
	command.sprintf( "ADG %lu ", ID );
	command += groupName.utf8() + " 0\r\n";
	socket->writeBlock( command, command.length() );
}

void KMService::renameGroup( QString groupName, QString group )
{
	groupName.replace( QRegExp( " " ), "%20" );
	ID = time( ( time_t * ) NULL );
	QString command;
	command.sprintf( "REG %lu ", ID );
	command += group;
	command += " " + groupName.utf8( ) + " 0\r\n";
	socket->writeBlock( command, command.length( ) );
}

void KMService::removeGroup( QString group )
{
	QString command;
	ID = time( ( time_t * ) NULL );
	command.sprintf( "RMG %lu ", ID );
	command += group + " \r\n";
	socket->writeBlock( command, command.length( ) );
}

/* Incoming RING command: connect to the Switchboard server and send the startChat signal */
// SessionID, Address, AuthInfo, handle, publicName     ( RNG)
void KMService::slotCreateChat( QString ID, QString address, QString auth, QString handle, QString publicName )
{
	chatService = new IMChatService();
	chatService->setHandle( myHandle );
	chatService->connectToSwitchBoard( ID, address, auth );
	emit startChat( chatService, handle );
}

/* 	Incoming XFR command: this is an result from slotStartChatSession(handle)
**	connect to the switchboard server and sen startChat signal */ 
// (XFR)
void KMService::slotCreateChat( QString address, QString auth )
{
	chatService = new IMChatService();
	chatService->setHandle( myHandle );
	chatService->msgHandle = msgHandle;
	chatService->connectToSwitchBoard( 0L, address, auth );
	emit startChat( chatService, msgHandle );
}

/*	Start a new chat session: the result is an XFR command, see above
**	**FIXME** The chat session needs an update, */
void KMService::slotStartChatSession( QString handle )
{
	/*  if(connected)
	    {
	      if( handle == myHandle)
	        return;*/
	msgHandle = handle;  //thats a problem, current implementation of chat session is a problem!!!! :(
	// **FIXME** next ugly code
	/*
	* save the handle, so class kmerlin can show the chatwindow
	*/
	theApp->calledHandle = handle;
	createChatSession( );
	// }
}
/**/
void KMService::createChatSession()
{
	ID = time( ( time_t * ) NULL );
	QString command;
	command.sprintf( "XFR %lu SB\r\n", ID );
	socket->writeBlock( command, command.length() );
}
/**/
void KMService::blockContact( const QString &handle, QString msnNick )
{
	removeContact( handle, "", "AL" ); // remove it from the al list
	addContact( handle, msnNick, "", "BL" );
}
/**/
void KMService::allowContact( const QString &handle, QString msnNick )
{
	removeContact( handle, "", "BL" );
	addContact( handle, msnNick, "", "AL" );
}
/**/
void KMService::removeContact( const QString &handle, const QString &group )
{
	removeContact( handle, group, "FL" );
}
/**/
void KMService::removeContactL( const QString &handle, const QString &list )
{
	removeContact( handle, "", list );
}
/**/
void KMService::addContact( const QString &handle, const QString &id )
{
	addContact( handle, handle, id, "FL" );
	addContact( handle, handle, "", "AL" );
}
/**/
void KMService::insertContact( const QString &handle, QString msnNick, const QString &group )
{
	addContact( handle, msnNick, group, "FL" );
}
void KMService::checkRemovedContacts()
{
	KMContact * contact;
	for ( contact = theApp->contactList.first(); contact != 0; contact = theApp->contactList.next() ) {
		if ( !contact->isReverse() && !contact->isForward() ) { // unsure
			contact->cleanUp();
		}
	}
}
void KMService::slotOpenInbox()
{
	KTempFile tmpFile( locateLocal( "tmp", "kmerlin" ), ".html" );
	QString UserID = myHandle;
	time_t now = time( ( time_t * ) NULL ) - loginTime;
	QString md5this( m_MSPAuth );
	md5this += QString( "%1" ).arg( now ) + myPassword ;
	KMD5 md5( md5this );
	m_hotmailRequest =
	    "<html>\n"
	    "<head>\n"
	    "<noscript>\n"
	    "<meta http-equiv=Refresh content=\"0; url=http://www.hotmail.com\">\n"
	    "</noscript>\n"
	    "</head>\n"
	    "<body onload=\"document.pform.submit(); \">\n"
	    "<form name=\"pform\" action=\"https://loginnet.passport.com/ppsecure/md5auth.srf?lc=1033\" method=\"POST\">\n"
	    "<input type=\"hidden\" name=\"mode\" value=\"ttl\">\n"
	    "<input type=\"hidden\" name=\"login\" value=\"" + UserID.left( UserID.find( '@' ) ) + "\">\n"
	    "<input type=\"hidden\" name=\"username\" value=\"" + UserID + "\">\n"
	    "<input type=\"hidden\" name=\"sid\" value=\"" + m_sid + "\">\n"
	    "<input type=\"hidden\" name=\"kv\" value=\"" + m_kv + "\">\n"
	    "<input type=\"hidden\" name=\"id\" value=\"2\">\n"
	    "<input type=\"hidden\" name=\"sl\" value=\"" + QString( "%1" ).arg( now ) + "\">\n"
	    "<input type=\"hidden\" name=\"rru\" value=\"/cgi-bin/HoTMaiL\">\n"
	    "<input type=\"hidden\" name=\"auth\" value=\"" + m_MSPAuth + "\">\n"
	    "<input type=\"hidden\" name=\"creds\" value=\"" + QString::fromLatin1( md5.hexDigest() ) + "\">\n"
	    "<input type=\"hidden\" name=\"svc\" value=\"mail\">\n"
	    "<input type=\"hidden\" name=\"js\" value=\"yes\">\n"
	    "</form></body>\n"
	    "</html>\n";
	//tmpFile.setAutoDelete(true);
	*tmpFile.textStream() << m_hotmailRequest;
	kapp->invokeBrowser( tmpFile.name() );
}

void KMService::authenticate()
{
	AuthDlg * auth = (AuthDlg*)sender()->parent()->parent();
	if ( auth->m_allow->isChecked() )
		addContact( auth->m_handle->text(), "0" );
	else{
		addContact( auth->m_handle->text(), "0" );
		blockContact( auth->m_handle->text(), auth->m_handle->text() );
	}
	delete auth;
}
/** sends a ping to the server- result is QNG */
void KMService::sendPing()
{
	lastPing = false;
	QString command = "PNG \r\n";
	socket->writeBlock( command, command.length() );
}
void KMService::slotPing()
{
	if ( connected ) {
		if ( lastPing ) { // ok ping again
			QTimer::singleShot( 60000, this, SLOT( slotPing() ) );
			sendPing();
			return ;
		} else { // connections lost;
			killSocket();
			return ;
		}
	}
}
